/// <reference types="node" />
import { Worker } from 'worker_threads';
export declare type Eworker<Input, Output> = (value: Input) => Output | Promise<Output>;
export interface IEworkOptions {
    /**
     * Maximum number of workers that will be spawned. This number must be at least
     * one and is only taken into account if smaller than the number of CPUs.
     * Default: number of logical CPUs.
     */
    maxWorkers?: number;
    /**
     * Minimum number of CPU threads that will be kept free.
     * Along with `maxWorkers`, this will determine the number of workers that will
     * be spawned. If there is only one CPU, this option is ignored and one worker
     * will be spawned.
     * Default: 1.
     */
    minFreeThreads?: number;
}
declare type WorkerResolveFn<Output> = (result: Output) => void;
declare type WorkerRejectFn = (reason: unknown) => void;
export interface IWorker<Input, Output> {
    worker: Worker;
    isWorking: boolean;
    job: IWorkerJob<Input, Output> | null;
}
interface IWorkerJob<Input, Output> {
    id: number;
    value: Input;
    resolve: WorkerResolveFn<Output>;
    reject: WorkerRejectFn;
}
export declare class Ework<Input, Output> {
    private totalWorkers;
    private freeWorkers;
    private workers;
    private queue;
    private nextWorkId;
    constructor(worker: Eworker<Input, Output>, options?: IEworkOptions);
    execute(value: Input): Promise<Output>;
    terminate(): Promise<void>;
    map(values: Input[]): Promise<Output[]>;
    private enqueue;
    private run;
    private getFreeWorker;
}
export {};
