"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const worker_threads_1 = require("worker_threads");
const cpuList = os_1.cpus();
exports.numCpus = cpuList ? cpuList.length : 1;
function spawnWorker(workerCode) {
    return new worker_threads_1.Worker(workerCode, { eval: true });
}
exports.spawnWorker = spawnWorker;
function terminateWorker(worker) {
    return new Promise((resolve) => {
        worker.worker.terminate(() => resolve());
        if (worker.job !== null) {
            worker.job.reject(new Error('worker terminated'));
        }
    });
}
exports.terminateWorker = terminateWorker;
function addWorkerListener(worker, type, listener) {
    worker.addListener(type, listener);
}
exports.addWorkerListener = addWorkerListener;
function makeWorkerCode(workerString) {
    return `'use strict';

const { parentPort } = require('worker_threads');

const workerFunction = ${workerString};

parentPort.on('message', async (message) => {
  if (message.type === 'work') {
    try {
      const result = await workerFunction(message.value);
      parentPort.postMessage({
        id: message.id,
        type: 'result',
        status: 'success',
        result
      });
    } catch (error) {
      let errorToTransfer = error;
      if (error) {
        if (error.stack) {
          errorToTransfer = error.stack;
        } else if (error.message) {
          errorToTransfer = error.message;
        }
      }
      try {
        parentPort.postMessage({
          id: message.id,
          type: 'result',
          status: 'error',
          error: errorToTransfer
        });
      } catch (e) {
        parentPort.postMessage({
          id: message.id,
          type: 'result',
          status: 'error',
          error: 'Work failed but error could not be transferred: ' + e.message
        });
      }
    }
  }
});
  `;
}
exports.makeWorkerCode = makeWorkerCode;
//# sourceMappingURL=worker.js.map